// Decal Shader for smoke and particles

float4x4 View : View;
float4x4 World : World;
float4x4 WorldView : WorldView;
float4x4 WorldViewProjection : WorldViewProjection;
float alphaoverride  : alphaoverride;
float4 clipPlane : ClipPlane;

float4 FogColor : Diffuse
<   string UIName =  "Fog Color";    
> = {0.0f, 0.0f, 0.0f, 0.0000001f};

float4 HudFogColor : Diffuse
<   string UIName =  "Hud Fog Color";    
> = {0.0f, 0.0f, 0.0f, 0.0000001f};

float4 HudFogDist : Diffuse
<   string UIName =  "Hud Fog Dist";    
> = {1.0f, 0.0f, 0.0f, 0.0000001f};

float4 UVScaling : uvscaling
<   string UIName =  "UV Scaling";    
> = {0, 0, 0, 0};

// RENDERCOLORTARGET simply indicates that we are using an RT (depth texture at bottom)
texture DiffuseMap : RENDERCOLORTARGET
<
    string Name = "D.tga";
    string type = "2D";
>;

sampler2D DiffuseSampler = sampler_state
{
    Texture   = <DiffuseMap>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = wrap; 
	AddressV = wrap;
};

struct appdata 
{
    float4 Position	: POSITION;
    float2 UV		: TEXCOORD0;    
};

struct vertexOutput
{
    float4 Position     : POSITION;
    float2 atlasUV      : TEXCOORD1; 
    float  WaterFog     : TEXCOORD2; 
    float4 WPos         : TEXCOORD3;
    float  clip         : TEXCOORD4;   
};

vertexOutput mainVS(appdata IN)   
{
    vertexOutput OUT;
    float4 worldSpacePos = mul(IN.Position, World);
    OUT.Position = mul(IN.Position, WorldViewProjection);
 	OUT.atlasUV = IN.UV + UVScaling.xy;
    float4 cameraPos = mul( worldSpacePos, View );
    float fogstrength = cameraPos.z * FogColor.w;
    OUT.WaterFog = min(fogstrength,1.0);
    OUT.clip = dot(worldSpacePos, clipPlane);    
    OUT.WPos = worldSpacePos;   
    return OUT;
}

float4 mainPS(vertexOutput IN) : COLOR
{
    float4 finalcolor;
    clip(IN.clip);
    
    float4 diffusemap = tex2D(DiffuseSampler,IN.atlasUV);
    float alpha = diffusemap.a * alphaoverride;    
    float4 result =  diffusemap;
    
    // calculate hud (scene) pixel-fog
    float4 cameraPos = mul(IN.WPos, View);
    float hudfogfactor = saturate((cameraPos.z- HudFogDist.x)/(HudFogDist.y - HudFogDist.x));
    
    // mix in HUD (scene) Fog with final color;
    float4 hudfogresult = lerp(result,HudFogColor,hudfogfactor);
    
    // and Finally add in any Water Fog   
    float4 waterfogresult = lerp(hudfogresult,FogColor,IN.WaterFog);   
    finalcolor=float4(waterfogresult.xyz,alpha);  
	
    return finalcolor;
}

vertexOutput mainVS_renderdepth(appdata IN)   
{
    vertexOutput OUT;
    OUT.Position = mul(IN.Position, WorldViewProjection);
 	OUT.atlasUV = IN.UV + UVScaling.xy;
    OUT.WaterFog = 0;
    OUT.clip = 0;                                                                   
    float4 worldSpacePos = mul(IN.Position, World);
	worldSpacePos.w = 7000-mul(IN.Position, WorldView).z;
    OUT.WPos = worldSpacePos;
    return OUT;
}

float4 mainPS_renderdepth(vertexOutput IN) : COLOR
{
	clip(-1);
	//clip(tex2D(DiffuseSampler,IN.atlasUV.xy).a-0.124);
	return IN.WPos.wxyz;
}

technique dx9textured
{
    pass RenderDepthPixelsPass
	<
		string RenderColorTarget = "[depthtexture]";
	>
    {
        // shaders
        VertexShader = compile vs_3_0 mainVS_renderdepth();
        PixelShader  = compile ps_3_0 mainPS_renderdepth();
        AlphaTestEnable = false;
    }
    pass MainPass
	<
		string RenderColorTarget = "";
	>
    {
        // shaders
        VertexShader = compile vs_3_0 mainVS();
        PixelShader  = compile ps_3_0 mainPS();
        CullMode = none;
        AlphaBlendEnable = true; 
        SrcBlend = srcalpha; 
		BlendOp = add;
		DestBlend = invsrcalpha;  
        Zenable=true;      
        ZWriteEnable = false;
        AlphaTestEnable = false;
    }
}
