// GUI Shader
float4x4 World : World;
float4x4 WorldViewProjection : WorldViewProjection;
float4 clipPlane : ClipPlane;

texture DiffuseMap : RENDERCOLORTARGET
<
    string Name = "D.tga";
    string type = "2D";
>;

sampler2D DiffuseSampler = sampler_state
{
    Texture   = <DiffuseMap>;
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = wrap; 
	AddressV = wrap;
};

struct appdata 
{
    float4 Position	: POSITION;
    float2 UV		: TEXCOORD0;    
};

struct vertexOutput
{
    float4 Position     : POSITION;
    float2 atlasUV      : TEXCOORD1; 
    float  clip         : TEXCOORD4;   
};

vertexOutput mainVS(appdata IN)   
{
    vertexOutput OUT;
    float4 worldSpacePos = mul(IN.Position, World);
    OUT.Position = mul(IN.Position, WorldViewProjection);
 	OUT.atlasUV = IN.UV;
    OUT.clip = dot(worldSpacePos, clipPlane);    
    return OUT;
}

float4 mainPS(vertexOutput IN) : COLOR
{
    float4 finalcolor;
    clip(IN.clip);
    finalcolor = tex2D(DiffuseSampler,IN.atlasUV);
    return finalcolor;
}

technique dx9textured
{
    pass MainPass
	<
		string RenderColorTarget = "";
	>
    {
        // shaders
        VertexShader = compile vs_3_0 mainVS();
        PixelShader  = compile ps_3_0 mainPS();
        CullMode = none;
        AlphaBlendEnable = true; 
        SrcBlend = srcalpha; 
		BlendOp = add;
		DestBlend = invsrcalpha;  
        Zenable = true;      
        ZWriteEnable = false;
        AlphaTestEnable = false;
    }
}
